(*
    Authors:    Jose Divasón
                Sebastiaan Joosten
                René Thiemann
                Akihisa Yamada
    License:    BSD
*)

section \<open>Example of executions\<close>


theory Example_Executions
  imports LLL_Factorization.Factorization_Algorithm_16_22
    "HOL-Library.Code_Char"
begin

text \<open>Conversions from Isabelle datatypes from and to integer lists which are available in
 target language.\<close>

definition list_to_basis :: "integer list list \<Rightarrow> int vec list"
  where "list_to_basis = map (vec_of_list o map int_of_integer)" 

definition vector_to_list :: "int vec \<Rightarrow> integer list" 
  where "vector_to_list = map integer_of_int o list_of_vec" 

definition basis_to_list :: "int vec list \<times> rat vec list \<Rightarrow> integer list list" 
  where "basis_to_list = map vector_to_list o fst" 

definition poly_to_list :: "int poly \<Rightarrow> integer list" 
  where "poly_to_list = map integer_of_int o coeffs" 

definition list_to_poly :: "integer list \<Rightarrow> int poly" 
  where "list_to_poly = poly_of_list o map int_of_integer" 

definition short_test :: "integer list list \<Rightarrow> integer list" where
  "short_test = vector_to_list o short_vector 2 o list_to_basis" 

definition weak_test :: "integer list list \<Rightarrow> integer list list" where
  "weak_test xs = (basis_to_list o weakly_reduce_basis (length xs) 2 o list_to_basis) xs" 

definition strict_test :: "integer list list \<Rightarrow> integer list list" where
  "strict_test xs = (basis_to_list o strictly_reduce_basis (length xs) 2 o list_to_basis) xs" 

definition LLL_factorization_test :: "integer list \<Rightarrow> integer list list" where
  "LLL_factorization_test xs = (map poly_to_list o LLL_factorization o list_to_poly) xs" 

definition factorization_algorithm_16_22_test :: "integer list \<Rightarrow> integer list list" where
  "factorization_algorithm_16_22_test xs = (map poly_to_list o factorization_algorithm_16_22 o list_to_poly) xs" 


(* example of textbook *)
value (code) "short_test [[12,2], [13,4]]" 

(* example in paper *)
lemma "short_test
    [[1, 1894885908, 0], 
     [0, 1, 1894885908], 
     [0, 0, 2147483648]]
 = [- 3, 17, 4]" by eval

lemma "vec_of_list [- 3, 17, 4] = 
    -3 \<cdot>\<^sub>v vec_of_list [1, 1894885908, 0] + 
    5684657741 \<cdot>\<^sub>v vec_of_list [0, 1, 1894885908] +
    -5015999938 \<cdot>\<^sub>v vec_of_list [0, 0, 2147483648 :: int]" 
  by eval

lemma "[[1, 1894885908, 0], [0, 1, 1894885908], [0, 0, 2147483648 :: int]] =
    map list_of_vec (factorization_lattice [:1894885908, 1:] 2 (2^31))" 
  by eval


(* example showing that the result may contain numbers (here: 13) 
   that are larger than the maximal input numbers (here: 10) *) 
lemma "short_test [[10,9,9], [3,-10,-10],[10,-10,10]] = [13, -1, -1]" 
  by eval

definition example_16_4_page_463 where 
  "example_16_4_page_463 = (let 
    n = 3;
    v1 = vec_of_list [1,1,0 :: rat];
    v2 = vec_of_list [1,0,1];
    v3 = vec_of_list [0,1,1];
    V = [v1,v2,v3];
    (M,W) = gram_schmidt_wit n V;
    MM = mat_of_rows_list n M;
    WW = mat_of_rows n W;
    VV = mat_of_rows n V
    in (map show V, map show M, map show W))" 
  
value (code) example_16_4_page_463
 
value (code) "let u = [:-34967,1 :: int :]; j = 1 + degree u; m = 117649 
  in show (weakly_reduce_basis 3 2 (factorization_lattice u j m))"
    
value (code) "let u = [:-34967,1 :: int :]; j = 3; m = 117649 
  in LLL_implementation.LLL_short_polynomial m j u"  

value (code) "LLL_factorization_test [-1,0,0,1]"
value (code) "LLL_factorization_test [-1,0,1]"
value (code) "LLL_factorization_test [1,1,1,1]"
value (code) "LLL_factorization_test [1,4,1,5,0,1]"
value (code) "LLL_factorization_test [4,5,15,5,6]"
value (code) "LLL_factorization_test             [4,5,3,2,4,8,2]"
value (code) "factorization_algorithm_16_22_test [4,5,3,2,4,8,2]"
value (code) "LLL_factorization_test             [4,0,0,2,1,3]"
value (code) "factorization_algorithm_16_22_test [4,0,0,2,1,3]"
value (code) "LLL_factorization_test             [-1,0,0,0,1]"
value (code) "factorization_algorithm_16_22_test [-1,0,0,0,1]"

text \<open>We now define the function that is used for the evaluation in the paper.
  It computes the squared norms of the short vector from an input lattice.\<close>

definition short_norm_test :: "integer list list \<Rightarrow> integer" where
  "short_norm_test = integer_of_int o sq_norm o short_vector 2 o list_to_basis" 

export_code short_norm_test strict_test weak_test short_norm_test 
  LLL_factorization_test factorization_algorithm_16_22_test
  in Haskell module_name LLL file "../generated" 

end
