section \<open>A list of definitions and theorems given in the submitted ITP paper\<close>

theory Paper
  imports LLL_Factorization.Factorization_Algorithm_16_22
begin

(* preparations to align paper presentation with actual formalization *)
lemma div_div: "1 / (a / b) = b / (a :: 'a :: field)" by simp

context gram_schmidt
begin
context fixes fs gs :: "'a vec list" and m :: nat
  assumes ass: "lin_indpt_list fs"
    "length fs = m" 
    "gram_schmidt n fs = gs" 
begin
lemma ass': "snd (gram_schmidt.main n fs) = gs"
proof -
  obtain gs' where snd: "snd (gram_schmidt.main n fs) = gs'" by auto
  from main_connect[OF ass(1,2) snd] ass(3) have gs': "gs' = gs" by auto
  with snd show ?thesis by auto
qed

lemmas ass = ass(1-2) ass'

lemmas gram_schmidt = gram_schmidt[OF ass, unfolded corthogonal_is_orthogonal, unfolded ass(2)]

lemma matrix_equality: "mat_of_rows n fs = mat m m (\<lambda>(i, j). \<mu> fs i j) * mat_of_rows n gs" 
  using matrix_equality[OF ass, unfolded M_def] by auto

lemma gram_schmidt_short_vector: assumes h: "h \<in> lattice_of fs - {0\<^sub>v n}"
  shows "\<exists> g\<^sub>i \<in> set gs. \<parallel>g\<^sub>i\<parallel>\<^sup>2 \<le> \<parallel>h\<parallel>\<^sup>2" 
proof -
  from gram_schmidt_short_vector[OF ass h] obtain i where "i < m" 
    and short: "sq_norm (gso fs i) \<le> sq_norm h" by auto
  from \<open>i < m\<close> main_connect[OF ass] have "gso fs i \<in> set gs" by auto
  with short show ?thesis by auto
qed

(* List of theorems and definitions from the paper *) 

(* page 3 *)
thm lin_indpt_list_def[of fs]
thm gram_schmidt

(* page 4 *)
thm matrix_equality 
thm is_projection_def
thm gram_schmidt_short_vector

(* page 5 *)
thm weakly_reduced_def
thm weakly_reduced_imp_short_vector[OF ass, of \<alpha> h]
end
end

context LLL
begin
context fixes \<alpha> :: rat
  assumes alpha: "\<alpha> \<ge> 4/3" 
begin
(* page 8 *)
thm LLL_invariant_def[of L \<alpha> "(i,Fs,Gs)" fs gs, unfolded split]
thm basis_reduction_add_row_i_im1[of L \<alpha> i Fs Gs fs gs i' fs' gs' mu]
thm gs.Gramian_determinant_def[of fs k] 
thm gs.Gramian_matrix_alt_def[of k fs]
thm LLL_measure_def[of \<alpha> "(i,fs,gs)", unfolded logD_def reduction_def of_rat_divide split div_div]

(* page 9 *)
thm Gramian_determinant[of L \<alpha> i Fs Gs fs gs k]
thm basis_reduction_step[OF alpha, of L i Fs Gs fs gs "(i',Fs',Gs')"]
thm LLL_measure_approx[OF alpha, of L i Fs Gs fs gs]
end

(* page 9 *)
thm short_vector[of \<alpha> fs v]
thm weakly_reduce_basis[of \<alpha> fs fs' gs']
thm strictly_reduce_basis[of \<alpha> fs fs' gs']
end

(* page 13 *)
thm Hadamard's_inequality[of A n]
thm common_factor_via_short (* lemma 1 in the paper, where all norms are squared *)
thm factorization_lattice_def

(* page 14 *)
thm LLL.lin_indpt_list_factorization_lattice[OF refl, of u "k + degree u" m, simplified]
thm sylvester_sub_poly[of u d q k c]
thm LLL.factorization_lattice[of u m "degree u + k" f, simplified]

(* page 15 *)
context LLL_implementation
begin
thm LLL_short_polynomial_def[of n u]
thm LLL_short_polynomial[of u n]
thm LLL_reconstruction.simps[of f us]
thm LLL_reconstruction[of f us gs F l]
end

(* page 16 *)
thm LLL_factorization_def[of f]
thm LLL_factorization[of f gs]

(* algorithms are executable *)
value (code) "LLL_factorization             [:- 335, 1218, - 102, - 1850, 9, 15:]" 
value (code) "factorization_algorithm_16_22 [:- 335, 1218, - 102, - 1850, 9, 15:]" 

end
